#
# RISK ASSORTIVITY METHODS
#
from typing import List

from emodpy_hiv.demographics.HIVDemographics import HIVDemographics


def set_risk_assortivities(demographics: HIVDemographics,
                           risk_assortivity_commercial=None,
                           risk_assortivity_informal=None,
                           risk_assortivity_marital=None,
                           risk_assortivity_transitory=None) -> None:
    def ra_matrix(v: float):
        matrix = None if v is None else [[v, 1 - v, 0], [1 - v, v, v], [0, v, 1 - v]]
        return matrix

    demographics.apply_assortivity(rel_type='COMMERCIAL',
                                   matrix=ra_matrix(v=risk_assortivity_commercial))

    demographics.apply_assortivity(rel_type='INFORMAL',
                                   matrix=ra_matrix(v=risk_assortivity_informal))

    demographics.apply_assortivity(rel_type='MARITAL',
                                   matrix=ra_matrix(v=risk_assortivity_marital))

    demographics.apply_assortivity(rel_type='TRANSITORY',
                                   matrix=ra_matrix(v=risk_assortivity_transitory))


#
# CONCURRENCY METHODS
#

def set_concurrency_parameters(demographics: HIVDemographics,
                               max_relationships_commercial_high_risk_male=None,
                               max_relationships_commercial_high_risk_female=None,
                               probability_extra_relationship_commercial_high_risk_male=None,
                               probability_extra_relationship_commercial_high_risk_female=None,

                               max_relationships_commercial_medium_risk_male=None,
                               max_relationships_commercial_medium_risk_female=None,
                               probability_extra_relationship_commercial_medium_risk_male=None,
                               probability_extra_relationship_commercial_medium_risk_female=None,

                               max_relationships_commercial_low_risk_male=None,
                               max_relationships_commercial_low_risk_female=None,
                               probability_extra_relationship_commercial_low_risk_male=None,
                               probability_extra_relationship_commercial_low_risk_female=None,

                               max_relationships_informal_high_risk_male=None,
                               max_relationships_informal_high_risk_female=None,
                               probability_extra_relationship_informal_high_risk_male=None,
                               probability_extra_relationship_informal_high_risk_female=None,

                               max_relationships_informal_medium_risk_male=None,
                               max_relationships_informal_medium_risk_female=None,
                               probability_extra_relationship_informal_medium_risk_male=None,
                               probability_extra_relationship_informal_medium_risk_female=None,

                               max_relationships_informal_low_risk_male=None,
                               max_relationships_informal_low_risk_female=None,
                               probability_extra_relationship_informal_low_risk_male=None,
                               probability_extra_relationship_informal_low_risk_female=None,

                               max_relationships_marital_high_risk_male=None,
                               max_relationships_marital_high_risk_female=None,
                               probability_extra_relationship_marital_high_risk_male=None,
                               probability_extra_relationship_marital_high_risk_female=None,

                               max_relationships_marital_medium_risk_male=None,
                               max_relationships_marital_medium_risk_female=None,
                               probability_extra_relationship_marital_medium_risk_male=None,
                               probability_extra_relationship_marital_medium_risk_female=None,

                               max_relationships_marital_low_risk_male=None,
                               max_relationships_marital_low_risk_female=None,
                               probability_extra_relationship_marital_low_risk_male=None,
                               probability_extra_relationship_marital_low_risk_female=None,

                               max_relationships_transitory_high_risk_male=None,
                               max_relationships_transitory_high_risk_female=None,
                               probability_extra_relationship_transitory_high_risk_male=None,
                               probability_extra_relationship_transitory_high_risk_female=None,

                               max_relationships_transitory_medium_risk_male=None,
                               max_relationships_transitory_medium_risk_female=None,
                               probability_extra_relationship_transitory_medium_risk_male=None,
                               probability_extra_relationship_transitory_medium_risk_female=None,

                               max_relationships_transitory_low_risk_male=None,
                               max_relationships_transitory_low_risk_female=None,
                               probability_extra_relationship_transitory_low_risk_male=None,
                               probability_extra_relationship_transitory_low_risk_female=None) -> None:

    demographics.set_concurrency_params_by_type_and_risk(rel_type="COMMERCIAL",
                                                         ip_value="HIGH",
                                                         max_simul_rels_male=max_relationships_commercial_high_risk_male,
                                                         max_simul_rels_female=max_relationships_commercial_high_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_commercial_high_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_commercial_high_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="COMMERCIAL",
                                                         ip_value="MEDIUM",
                                                         max_simul_rels_male=max_relationships_commercial_medium_risk_male,
                                                         max_simul_rels_female=max_relationships_commercial_medium_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_commercial_medium_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_commercial_medium_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="COMMERCIAL",
                                                         ip_value="LOW",
                                                         max_simul_rels_male=max_relationships_commercial_low_risk_male,
                                                         max_simul_rels_female=max_relationships_commercial_low_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_commercial_low_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_commercial_low_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="INFORMAL",
                                                         ip_value="HIGH",
                                                         max_simul_rels_male=max_relationships_informal_high_risk_male,
                                                         max_simul_rels_female=max_relationships_informal_high_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_informal_high_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_informal_high_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="INFORMAL",
                                                         ip_value="MEDIUM",
                                                         max_simul_rels_male=max_relationships_informal_medium_risk_male,
                                                         max_simul_rels_female=max_relationships_informal_medium_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_informal_medium_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_informal_medium_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="INFORMAL",
                                                         ip_value="LOW",
                                                         max_simul_rels_male=max_relationships_informal_low_risk_male,
                                                         max_simul_rels_female=max_relationships_informal_low_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_informal_low_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_informal_low_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="MARITAL",
                                                         ip_value="HIGH",
                                                         max_simul_rels_male=max_relationships_marital_high_risk_male,
                                                         max_simul_rels_female=max_relationships_marital_high_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_marital_high_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_marital_high_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="MARITAL",
                                                         ip_value="MEDIUM",
                                                         max_simul_rels_male=max_relationships_marital_medium_risk_male,
                                                         max_simul_rels_female=max_relationships_marital_medium_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_marital_medium_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_marital_medium_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="MARITAL",
                                                         ip_value="LOW",
                                                         max_simul_rels_male=max_relationships_marital_low_risk_male,
                                                         max_simul_rels_female=max_relationships_marital_low_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_marital_low_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_marital_low_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="TRANSITORY",
                                                         ip_value="HIGH",
                                                         max_simul_rels_male=max_relationships_transitory_high_risk_male,
                                                         max_simul_rels_female=max_relationships_transitory_high_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_transitory_high_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_transitory_high_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="TRANSITORY",
                                                         ip_value="MEDIUM",
                                                         max_simul_rels_male=max_relationships_transitory_medium_risk_male,
                                                         max_simul_rels_female=max_relationships_transitory_medium_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_transitory_medium_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_transitory_medium_risk_female)

    demographics.set_concurrency_params_by_type_and_risk(rel_type="TRANSITORY",
                                                         ip_value="LOW",
                                                         max_simul_rels_male=max_relationships_transitory_low_risk_male,
                                                         max_simul_rels_female=max_relationships_transitory_low_risk_female,
                                                         prob_xtra_rel_male=probability_extra_relationship_transitory_low_risk_male,
                                                         prob_xtra_rel_female=probability_extra_relationship_transitory_low_risk_female)


#
# PAIR FORMATION RATE METHODS
#

def set_pair_formation_rates(demographics: HIVDemographics,
                             pair_formation_rate_commercial=None,
                             pair_formation_rate_informal=None,
                             pair_formation_rate_marital=None,
                             pair_formation_rate_transitory=None) -> None:
    demographics.set_pair_form_params(rel_type="COMMERCIAL", new_constant_rate=pair_formation_rate_commercial)
    demographics.set_pair_form_params(rel_type="INFORMAL", new_constant_rate=pair_formation_rate_informal)
    demographics.set_pair_form_params(rel_type="MARITAL", new_constant_rate=pair_formation_rate_marital)
    demographics.set_pair_form_params(rel_type="TRANSITORY", new_constant_rate=pair_formation_rate_transitory)


#
# COITAL ACT RATE METHODS
#

def set_coital_act_rates(demographics: HIVDemographics,
                         coital_act_rate_commercial=None,
                         coital_act_rate_informal=None,
                         coital_act_rate_marital=None,
                         coital_act_rate_transitory=None) -> None:
    demographics.set_coital_act_rate(rel_type="COMMERCIAL", rate=coital_act_rate_commercial)
    demographics.set_coital_act_rate(rel_type="INFORMAL", rate=coital_act_rate_informal)
    demographics.set_coital_act_rate(rel_type="MARITAL", rate=coital_act_rate_marital)
    demographics.set_coital_act_rate(rel_type="TRANSITORY", rate=coital_act_rate_transitory)


#
# CONDOM USAGE METHODS
#

def set_condom_usage_probabilities(demographics: HIVDemographics,
                                   condom_usage_min_commercial=None,
                                   condom_usage_mid_commercial=None,
                                   condom_usage_max_commercial=None,
                                   condom_usage_rate_commercial=None,

                                   condom_usage_min_informal=None,
                                   condom_usage_mid_informal=None,
                                   condom_usage_max_informal=None,
                                   condom_usage_rate_informal=None,

                                   condom_usage_min_marital=None,
                                   condom_usage_mid_marital=None,
                                   condom_usage_max_marital=None,
                                   condom_usage_rate_marital=None,

                                   condom_usage_min_transitory=None,
                                   condom_usage_mid_transitory=None,
                                   condom_usage_max_transitory=None,
                                   condom_usage_rate_transitory=None) -> None:
    demographics.set_condom_usage_probs(rel_type="COMMERCIAL",
                                        min=condom_usage_min_commercial, mid=condom_usage_mid_commercial,
                                        max=condom_usage_max_commercial, rate=condom_usage_rate_commercial)

    demographics.set_condom_usage_probs(rel_type="INFORMAL",
                                        min=condom_usage_min_informal, mid=condom_usage_mid_informal,
                                        max=condom_usage_max_informal, rate=condom_usage_rate_informal)

    demographics.set_condom_usage_probs(rel_type="MARITAL",
                                        min=condom_usage_min_marital, mid=condom_usage_mid_marital,
                                        max=condom_usage_max_marital, rate=condom_usage_rate_marital)

    demographics.set_condom_usage_probs(rel_type="TRANSITORY",
                                        min=condom_usage_min_transitory, mid=condom_usage_mid_transitory,
                                        max=condom_usage_max_transitory, rate=condom_usage_rate_transitory)


#
# RELATIONSHIP DURATION METHODS
#

def set_relationship_durations(demographics: HIVDemographics,
                               commercial_weibull_heterogeneity=None,
                               commercial_weibull_scale=None,

                               informal_weibull_heterogeneity=None,
                               informal_weibull_scale=None,

                               marital_weibull_heterogeneity=None,
                               marital_weibull_scale=None,

                               transitory_weibull_heterogeneity=None,
                               transitory_weibull_scale=None) -> None:
    demographics.set_relationship_duration(rel_type="COMMERCIAL",
                                           weibull_heterogeneity=commercial_weibull_heterogeneity,
                                           weibull_scale=commercial_weibull_scale)

    demographics.set_relationship_duration(rel_type="INFORMAL",
                                           weibull_heterogeneity=informal_weibull_heterogeneity,
                                           weibull_scale=informal_weibull_scale)

    demographics.set_relationship_duration(rel_type="MARITAL",
                                           weibull_heterogeneity=marital_weibull_heterogeneity,
                                           weibull_scale=marital_weibull_scale)

    demographics.set_relationship_duration(rel_type="TRANSITORY",
                                           weibull_heterogeneity=transitory_weibull_heterogeneity,
                                           weibull_scale=transitory_weibull_scale)


#
# INITIAL RISK DISTRIBUTION METHODS
#

def set_initial_risk_distribution(demographics: HIVDemographics, initial_risk_distribution_low: float = None) -> None:
    def risk_distribution(low: float):
        return None if low is None else [low, 1 - low, 0]

    distribution = risk_distribution(low=initial_risk_distribution_low)
    demographics.add_or_update_initial_risk_distribution(distribution=distribution)
